﻿define([
    "dojo/_base/declare",
    "epi/shell/command/_Command",

    "epi/shell/TypeDescriptorManager",

    "epi-cms/core/ContentReference",

    "epi/dependency",


    // language resources
    "epi/i18n!epi/cms/nls/bynder",

    "https://ucv.bynder.com/5.0.5/modules/compactview/bynder-compactview-3-latest.js"
],
    function (
        declare,
        _Command,
        TypeDescriptorManager,
        ContentReference,
        dependency,
        language
    ) {

        return declare("bynder.commands.BynderAddAssetCommand", [_Command], {
            iconClass: "dijitNoIcon",
            label: "",
            moduleArea: "bynder",
            canExecute: true,
            isAvailable: true,
            settings: null,

            _store: null,

            postscript: function () {
                this.inherited(arguments);

                var registry = dependency.resolve("epi.storeregistry");
                this._store = registry.get("BynderAssetMappingStore");

                this.label = language.command.provider.bynder.label;
            },

            onExecuted: function (contentReference) {
                // callback
                // thumbnail image in edit mode will be empty since we don't inherit from ImageData
            },

            _onModelChange: function () {
                if (!this.model || !this.settings.availableTypes) {
                    return;
                }

                var types = TypeDescriptorManager.getValidAcceptedTypes(this.settings.availableTypes.split(","), this.model.allowedTypes, this.model.restrictedTypes);
                if (types && types.length > 0) {
                    // if we only have imagedata then update compact view type to filter on image
                    if (types.length === 1 && types[0] === "episerver.core.imagedata") {
                        this.settings.options.assetTypes = "image";
                    }

                    // if we only have videodata then update compact view type to filter on video
                    if (types.length === 1 && types[0] === "episerver.core.videodata") {
                        this.settings.options.assetTypes = "video";
                    }

                    return;
                }

                this.set("isAvailable", false);
            },

            _onCompactViewSuccess: function (assets, additionalInfo) {
                var asset = assets[0];
                var selectedDerivative = additionalInfo.selectedFile.url;

                asset.imageUrl = additionalInfo.selectedFile.url;

                this._store.query({ assetId: asset.databaseId, derivative: selectedDerivative }) // Since it's SingleSelectFile mode we're passing in the selected url as the derivative
                    .then(function (data) {
                        var value = new ContentReference(data.contentLink).toString();
                        if (this.model && this.model.modelType && this.model.modelType === "EPiServer.Cms.Shell.UI.ObjectEditing.InternalMetadata.LinkModel") {
                            value = { href: data.permanentLink };
                        }
                        this.onExecuted(value);
                    }.bind(this),
                        function (error) {
                            console.error(error);
                        }.bind(this));
            },

            _execute: function () {
                var that = this;

                var options = this.settings.options;

                BynderCompactView.open({
                    language: options.language,
                    defaultSearchTerm: options.searchTerm,
                    mode: "SingleSelectFile",
                    assetTypes: options.assetTypes.split(','),
                    portal: {
                        url: options.defaultEnvironment
                    },
                    onSuccess: function (assets, additionalInfo) {
                        that._onCompactViewSuccess(assets, additionalInfo);
                    }
                });
            }
        });
    });
