﻿define([
    // Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/promise/all",
    "dojo/store/Memory",

    // Dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_WidgetBase",
    "dijit/form/Button",
    //CMS
    "epi/dependency",
    "epi-cms/_ContentContextMixin",
    "epi/shell/widget/_ActionProviderWidget",
    // Episerver dnd classes
    "epi/shell/TypeDescriptorManager",
    // dgrid
    "dgrid/OnDemandGrid",
    "dgrid/extensions/DnD",

    "bynder/DerivativeHelper",

    "dojo/text!./BynderCompactViewComponent.html",
    "dojo/text!../Template/BynderError.html",
    "xstyle/css!./BynderCompactViewComponent.css",
    "https://ucv.bynder.com/5.0.5/modules/compactview/bynder-compactview-3-latest.js"
], function (
    // Dojo
    declare,
    lang,
    array,
    domConstruct,
    domClass,
    all,
    Memory,

    // Dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _WidgetBase,
    Button,
    //CMS
    dependency,
    _ContentContextMixin,
    _ActionProviderWidget,
    // Episerver dnd classes
    TypeDescriptorManager,
    // dgrid
    OnDemandGrid,
    DnD,

    DerivativeHelper,

    template,
    errorTemplate
) {
    return declare([_WidgetBase, _TemplatedMixin, _ContentContextMixin], {
        templateString: template,

        _memoryStore: null,

        button: null,
        _button: null,

        clearButton: null,
        _clearButton: null,

        source: null,
        _source: null,

        assetGrid: null,
        _assetGrid: null,

        compactViewOptions: null,
        allowedDerivatives: null,

        postCreate: function () {
            this.inherited(arguments);

            this._memoryStore = new Memory({
                data: [],
                idProperty: "contentLink",
            });

            var registry = dependency.resolve("epi.storeregistry");
            this._store = registry.get("BynderAssetMappingStore");

            this._setupButtons();
            this._setupGrid();

            this.errorPanel.innerHTML = errorTemplate;
        },

        _setupButtons: function () {
            this._button = new Button({
                label: "Select assets",
                class: 'epi-flat epi-chromeless',
                iconClass: "epi-iconPlus",
                onClick: dojo.hitch(this, this._onButtonClick)
            },
                this.button);
            this._button.startup();

            this._clearButton = new Button({
                label: "Clear",
                class: 'epi-flat epi-chromeless',
                iconClass: "epi-iconRevert",
                onClick: dojo.hitch(this, this._onClearButtonClick)
            },
                this.clearButton);
            this._clearButton.startup();
        },

        _setupGrid: function () {
            this._assetGrid = new declare([OnDemandGrid, DnD])(
                {
                    class: "epi-bynder-grid",
                    store: this._memoryStore,
                    showHeader: false,
                    selectionMode: "single",
                    sort: "default",
                    dndParams: {
                        copyOnly: true,
                        selfAccept: false,
                        accept: [],
                        creator: this._createAvatarItem.bind(this),
                    },
                    getObjectDndType: this._getDndTypes.bind(this),
                    columns: [
                        {
                            field: "bynderImage",
                            label: "Image",
                            renderCell: function (item) {
                                return domConstruct.create("img", {
                                    src: item.properties.imageUrl,
                                    class: "epi-bynder-thumbnail",
                                });
                            },
                        },
                        {
                            field: "name",
                            label: "Name",
                        },
                    ],
                },
                this.assetGrid
            );

            this._assetGrid.refresh();
        },

        _onButtonClick: function () {
            var that = this;

            BynderCompactView.open({
                language: this.compactViewOptions.language,
                defaultSearchTerm: this.compactViewOptions.searchTerm,
                mode: "MultiSelect",
                assetTypes: this.compactViewOptions.assetTypes.split(','),
                portal: {
                    url: this.compactViewOptions.defaultEnvironment
                },
                onSuccess: function (assets, additionalInfo) {
                    that._renderAssets(assets);
                }
            });
        },

        _onClearButtonClick: function () {
            this._memoryStore.setData([]);
            this._assetGrid.refresh();
        },

        _renderAssets: function (assets) {
            if (!assets || !assets.length) {
                return;
            }

            var asset = assets.pop();
            this._renderAsset(asset).then(
                function () {
                    this._renderAssets(assets);
                }.bind(this)
            );
        },

        _renderAsset: function (asset) {
            var derivatives = DerivativeHelper.getAvailableDerivatives(asset, this.allowedDerivatives);
            var getContentPromises = array.map(
                derivatives,
                function (derivative) {
                    return this._store.query({
                        assetId: asset.databaseId,
                        derivative: derivative,
                    });
                },
                this
            );
            var allPromises = all(getContentPromises);
            allPromises.then(
                lang.hitch(this, function (contentList) {
                    this._updateValue(contentList);
                }),
                lang.hitch(this, function (error) {
                    domClass.add(this.bynderComponent, "epi-bynder--has-error");
                    console.error(error);
                })
            );
            return allPromises;
        },

        _updateValue: function (contentList) {
            array.forEach(
                contentList,
                function (contentItem) {
                    this._memoryStore.put(contentItem);
                },
                this
            );
            this._assetGrid.refresh();
        },

        _createAvatarItem: function (item) {
            //This node is used when dragging the item.
            return {
                node: domConstruct
                    .create("div")
                    .appendChild(domConstruct.create("img", {
                        src: item.properties.imageUrl,
                        class: "epi-bynder-thumbnail--large",
                    })),
                data: item,
            };
        },

        _getDndTypes: function (item) {
            return TypeDescriptorManager.getAndConcatenateValues(item.typeIdentifier, "dndTypes");
        },
    });
});
