﻿define("bynder/DerivativeHelper", ["dojo/_base/array"], function (array) {
    return {
        getAvailableDerivatives: function (asset, allowedDerivatives) {
            // In CompactView 2.0 and later not all derivatives are listed under asset.derivatives, the remaining can be found under asset.files
            this._fixMissingDerivatives(asset);

            if (asset.type === "VIDEO" || asset.type === "DOCUMENT" || asset.type === "AUDIO") {
                return [asset.type];
            }

            var allDerivatives = Object.keys(asset.derivatives).map(key => key.toLowerCase());

            // lower casing before comparing
            if (typeof allowedDerivatives === 'string' || allowedDerivatives instanceof String) {
                allowedDerivatives = allowedDerivatives?.toLowerCase();
            }
            else if (Array.isArray(allowedDerivatives)) {
                for (var i = 0; i < allowedDerivatives.length; i++) {
                    allowedDerivatives[i] = allowedDerivatives[i].toString().toLowerCase();
                }
            }

            if (!allowedDerivatives) {
                return allDerivatives;
            }

            return array.filter(
                allDerivatives,
                function (derivative) {
                    return allowedDerivatives.indexOf(derivative) !== -1;
                },
                this
            );
        },
        _fixMissingDerivatives: function (asset) {
            if (!asset.files) {
                return;
            }

            var allFiles = Object.keys(asset.files);
            var allDerivatives = Object.keys(asset.derivatives);

            allFiles = allFiles.filter(function (file) {
                return allDerivatives.indexOf(file) === -1;
            });

            allFiles.forEach(function (file) {
                asset.derivatives[file] = asset.files[file].url;
            });
        }
    };
});
