﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-class",

    "dijit/_CssStateMixin",
    "dijit/_Widget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",

    "epi/dependency",
    "epi/shell/widget/_ValueRequiredMixin",
    "epi/shell/widget/_ActionProviderWidget",

    "bynder/DerivativeHelper",

    "dojo/text!./BynderAssetSelectorDialog.html",
    "dojo/text!../Template/BynderError.html",

    "dijit/MenuItem",
    "xstyle/css!./BynderAssetSelectorDialog.css",
    "https://ucv.bynder.com/5.0.5/modules/compactview/bynder-compactview-3-latest.js"
], function (
    declare,
    lang,
    domConstruct,
    domClass,

    _CssStateMixin,
    _Widget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,

    dependency,
    _ValueRequiredMixin,
    _ActionProviderWidget,

    DerivativeHelper,

    template,
    errorTemplate,

    MenuItem,
) {
    return declare(
        "bynder.editors.BynderAssetSelectorDialog",
        [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionProviderWidget],
        {
            templateString: template,
            intermediateChanges: false,
            _asset: null,
            _store: null,
            _script: null,
            _eventListener: null,

            _compactViewOptions: null,
            _allowedDerivatives: null,
            _contentLink: null,

            constructor: function (compactViewOptions, allowedDerivatives, contentLink) {
                this._compactViewOptions = compactViewOptions;
                this._allowedDerivatives = allowedDerivatives;
                this._contentLink = contentLink;
            },

            postCreate: function () {
                // call base implementation
                this.inherited(arguments);

                var registry = dependency.resolve("epi.storeregistry");
                this._store = registry.get("BynderAssetMappingStore");

                var assetId = '';

                if (this._contentLink) {

                    var request = new XMLHttpRequest();
                    request.open('GET', '/bynderapi/getexistingasset?contentLink=' + encodeURIComponent(this._contentLink), false);  // `false` makes the request synchronous
                    request.send(null);

                    if (request.status === 200) {
                        var asset = JSON.parse(request.responseText);
                        assetId = asset.databaseId;

                        var derivatives = DerivativeHelper.getAvailableDerivatives(
                            asset,
                            this._allowedDerivatives
                        );
                        var selectedDerivative = derivatives[0];
                        if (asset.selectedDerivative) {
                            selectedDerivative = asset.selectedDerivative;
                        }

                        this.derivativeDropDownButton.set("label", selectedDerivative);

                        this._buildMetaDataGrid(asset, selectedDerivative);

                        this._store.query({ assetId: asset.databaseId, derivative: selectedDerivative })
                            .then(function (data) {
                                this._renderAsset(data, asset, derivatives);
                            }.bind(this),
                                function (error) {
                                    this._renderError(error);
                                }.bind(this));
                    }
                }

                this._loadCompactView(assetId);

                // if previous selected asset is empty or mode is SingleSelectFile open compact view directly
                if (assetId === '' || this._compactViewOptions.mode === "SingleSelectFile") {
                    this._onCompactViewClick(assetId);
                }

                this.errorPanel.innerHTML = errorTemplate;
            },

            destroy: function () {
                // summary:
                //      Destroy the widget and hide the dialog.
                // tags:
                //      protected

                this.inherited(arguments);
            },

            _onCompactViewSuccess: function (assets, additionalInfo) {
                // Close CV after selection
                this.compactView = null;

                this.assetContainer.innerHTML = "<h2>Loading...</h2>";

                var asset = assets[0];

                var derivatives = null;
                var selectedDerivative = null;

                if (additionalInfo.selectedFile === undefined) {
                    derivatives = DerivativeHelper.getAvailableDerivatives(
                        asset,
                        this._allowedDerivatives
                    );

                    selectedDerivative = derivatives[0];

                    this.derivativeDropDownButton.set("label", selectedDerivative);
                    this._buildMetaDataGrid(asset, selectedDerivative);

                    if (derivatives.length === 0 || !selectedDerivative) {
                        this.assetContainer.innerHTML =
                            "<h2>Asset does not have any of the allowed derivatives: " + this._allowedDerivatives + "</h2>";
                        return;
                    }
                } 
                else { // If we're using SingleSelectFile mode the derivative is bypassed and we use the singleSelectFile.url property instead
                    selectedDerivative = additionalInfo.selectedFile.url; // inner object with the actual url
                    asset.imageUrl = selectedDerivative;
                }

                this._store.query({ assetId: asset.databaseId, derivative: selectedDerivative })
                    .then(function (data) {
                        this._renderAsset(data, asset, derivatives);

                    }.bind(this),
                        function (error) {
                            this._renderError(error);
                        }.bind(this));
            },

            _loadCompactView: function (assetId) {

                // Create button
                this.compactView = domConstruct.create('button',
                    {
                        id: 'bynder-compactview'
                    });

                this.compactView.setAttribute("data-assetid", assetId);

                this.compactView.innerText = this._compactViewOptions["button"];
                this._clickEventListener = lang.hitch(this, this._onCompactViewClick);
                this.compactView.addEventListener("click", this._clickEventListener);

                // Place button in DOM
                domConstruct.place(this.compactView, this.assetContainer, 'before');
            },

            _buildMetaDataGrid: function (asset, selectedDerivative) {
                this.derivativeMetaDataGrid.refresh();

                const gridData = [
                    { key: "Name", value: asset.name },
                    { key: "Type", value: asset.type }
                ];
                this.derivativeMetaDataGrid.renderArray(gridData);
            },

            _onSelectDerivative: function (asset, evt) {
                const derivative = evt.srcElement.innerText;
                this.derivativeDropDownButton.set("label", derivative);
                dojo.when(
                    this._store.query({ assetId: asset.databaseId, derivative: derivative }),
                    lang.hitch(this, function (data) {
                        this._updateValue(data.contentLink, asset);
                    })
                );
            },

            _renderAsset: function (data, asset, derivatives) {
                this.assetContainer.innerHTML = "";

                this.derivativeContainer.style.display = 'none';

                if (this.derivativeDropDownMenu.hasChildren()) {
                    var items = this.derivativeDropDownMenu.getChildren();

                    for (var i = 0; i < items.length; i++) {
                        this.derivativeDropDownMenu.removeChild(items[i]);
                    }
                }

                this._updateValue(data.contentLink, asset);

                switch (asset.type.toUpperCase()) {
                    case "AUDIO":
                    case "DOCUMENT":
                    case "IMAGE":
                        
                        if (asset.imageUrl) { // This will use the image selected in Compact View
                            this.assetContainer.innerHTML +=
                                "<img src='" +
                                asset.imageUrl +
                                "' title='" +
                                asset.name +
                                "' style='max-width: 600px' />";
                        }
                        else if (asset.derivatives.thumbnail) {
                            this.assetContainer.innerHTML +=
                                "<img src='" +
                                asset.derivatives.thumbnail +
                                "' title='" +
                                asset.name +
                                "' />";
                        }

                        if (derivatives == null || derivatives.length === 0) {
                            break;
                        }

                        for (var thumbnail in asset.derivatives) {
                            thumbnail = thumbnail.toLowerCase();
                            if (derivatives.indexOf(thumbnail) !== -1) {
                                this.derivativeDropDownMenu.addChild(
                                    new MenuItem({
                                        label: thumbnail,
                                        onClick: lang.hitch(this,
                                            function (evt) {
                                                this._onSelectDerivative(asset, evt);
                                            }),
                                    })
                                );
                            }
                        }

                        this.derivativeContainer.style.display = 'block';

                        break;
                    case "VIDEO":
                        this.assetContainer.innerHTML +=
                            '<video height="150" controls>' +
                            '<source src="' +
                            asset.previewUrls[0] +
                            '" type="video/mp4">' +
                            "</video>";
                        break;
                }
            },

            _renderError: function (error) {
                domClass.add(this.inputContainer, "epi-bynder--has-error");
                console.error(error);
            },

            _updateValue: function (value, asset) {
                this.set("value", value);
                this.set("asset", asset);
            },

            _onCompactViewClick: function (assetId) {
                var options = this._compactViewOptions;
                var that = this;

                var button = document.getElementById('bynder-compactview');

                if (button) {
                    assetId = button.getAttribute("data-assetid");
                }

                BynderCompactView.open({
                    language: options.language,
                    defaultSearchTerm: options.searchTerm,
                    mode: options.mode,
                    assetTypes: options.assetTypes.split(','),
                    selectedAssets: assetId.split(','),
                    portal: {
                        url: options.defaultEnvironment
                    },
                    onSuccess: function (assets, additionalInfo) {
                        that._onCompactViewSuccess(assets, additionalInfo); // additionalInfo is only passed when mode is SingleSelectFile
                    }
                });
            },

            focus: function () { },
        }
    );
});
